/*
 * Common font-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdlib.h>

#include "display.h"
#include "font.h"



/* Initialise a font configuration. */

void init_font(font_config_t *cfg, uint8_t *chars, uint32_t *table,
                                   uint32_t base, uint32_t limit)
{
    cfg->chars = (char_definition_t *) chars;
    cfg->table = table;
    cfg->base = base;
    cfg->limit = limit;
}



/* Return the character definition for the given character. */

char_definition_t *get_char_definition(font_config_t *fcfg, char c)
{
    /* Obtain the offset from the table and combine it with the character
       definitions base address. */

    if ((c >= fcfg->base) && (c < fcfg->limit))
        return (char_definition_t *) ((uint8_t *) fcfg->chars + fcfg->table[c - fcfg->base]);
    else
        return NULL;
}

/* Write a character to the screen, returning the next character position. */

int write_char(display_config_t *cfg, font_config_t *fcfg, char c,
               int x, int y, uint8_t colour)
{
    int sy, dx, dy;
    char_definition_t *chardef = get_char_definition(fcfg, c);
    uint8_t *imagerow = chardef->data,
            *displayline = cfg->screen_start + y * cfg->line_length,
            bitmap, mask;

    /* Space is a special case. */

    if (c == ' ')
        return x + 3;

    for (sy = 0, dy = y; (sy < chardef->height) && (dy < cfg->line_count); sy++, dy++)
    {
        if (displayline >= cfg->screen_limit)
            displayline -= cfg->screen_size;

        bitmap = *imagerow;

        for (dx = x, mask = 1 << (chardef->width - 1); (mask) && (dx < cfg->line_length); dx++, mask >>= 1)
        {
            if (bitmap & mask)
                displayline[display_get_position(cfg, dx)] = colour;
        }

        imagerow += 1;
        displayline += cfg->line_length;
    }

    return x + chardef->width;
}
