/*
 * PIC32 microcontroller initialisation code.
 *
 * Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "mips.h"
#include "pic32.h"

/* Application-specific configuration. */

#include "devconfig.h"

/* Disable JTAG functionality on pins. */

.section .devcfg0, "a"
.word 0xfffffffb	/* DEVCFG0<2> = JTAGEN = 0 */

/* Select oscillator, pin usage, watchdog timer and peripheral clock divider. */

.section .devcfg1, "a"
.word (DEVCFG1_UNUSED | DEVCFG1_CONFIG)

/* Set clock dividers and multiplier. */

.section .devcfg2, "a"
.word (DEVCFG2_UNUSED | DEVCFG2_CONFIG)

/* The start routine is placed at the boot location. */

.section .boot, "a"

.globl _start
.extern relocate_data
.extern init_bss
.extern main

_start:
	/* Enable caching. */

	mfc0 $v1, CP0_CONFIG
	li $t8, ~CONFIG_K0
	and $v1, $v1, $t8
	ori $v1, $v1, CONFIG_K0_CACHABLE_NONCOHERENT
	mtc0 $v1, CP0_CONFIG
	nop

	/* Get the RAM size. */

	la $v1, BMXDRMSZ
	lw $t0, 0($v1)

        /* Initialise the stack pointer. */

        li $v1, KSEG0_BASE
        addu $sp, $t0, $v1      /* sp = KSEG0_BASE + RAM size */

        /* Initialise the globals pointer. */

        lui $gp, %hi(_GLOBAL_OFFSET_TABLE_)
        ori $gp, $gp, %lo(_GLOBAL_OFFSET_TABLE_)

	/* Clear or relocate mutable data and then jump to the main program. */

	la $t9, init_bss
	jalr $t9
	nop

	la $t9, relocate_data
	jalr $t9
	nop

	la $t9, main
	jalr $t9
	nop

_end:
	b _end
	nop
